@@ -1,16 +1,12 @@
 import json
 import time
-from typing import Callable, Iterator, Optional
+from typing import Callable, Optional
 
 import pandas as pd
-from dagster_pandas import DataFrame
-
 from dagster import (
     Array,
-    AssetMaterialization,
     Bool,
     DagsterInvalidDefinitionError,
-    EventMetadataEntry,
     Failure,
     Field,
     InputDefinition,
@@ -25,151 +21,101 @@ from dagster import (
     check,
     solid,
 )
+from dagster.core.execution.context.compute import SolidExecutionContext
+from dagster_pandas import DataFrame
 
-from ..errors import DagsterDbtUnexpectedRpcPollOutput
-from .types import DbtRpcPollResult
+from ..errors import DagsterDbtRpcUnexpectedPollOutputError
+from .types import DbtRpcOutput
 from .utils import log_rpc, raise_for_rpc_error
 
 
-def generate_materializations(rpc_poll_result: DbtRpcPollResult) -> Iterator[AssetMaterialization]:
-    for node_result in rpc_poll_result.results:
-        if node_result.node["resource_type"] in ["model", "snapshot"]:
-            success = not node_result.fail and not node_result.skip and not node_result.error
-            if success:
-                entries = [
-                    EventMetadataEntry.json(data=node_result.node, label="Node"),
-                    EventMetadataEntry.text(text=node_result.status, label="Status"),
-                    EventMetadataEntry.text(
-                        text=str(node_result.execution_time), label="Execution Time"
-                    ),
-                    EventMetadataEntry.text(
-                        text=node_result.node["config"]["materialized"],
-                        label="Materialization Strategy",
-                    ),
-                    EventMetadataEntry.text(text=node_result.node["database"], label="Database"),
-                    EventMetadataEntry.text(text=node_result.node["schema"], label="Schema"),
-                    EventMetadataEntry.text(text=node_result.node["alias"], label="Alias"),
-                    EventMetadataEntry.text(
-                        text=node_result.node["description"], label="Description"
-                    ),
-                ]
-                for node_timing in node_result.timing:
-                    if node_timing.name == "execute":
-                        execution_entries = [
-                            EventMetadataEntry.text(
-                                text=node_timing.started_at.isoformat(timespec="seconds"),
-                                label="Execution Started At",
-                            ),
-                            EventMetadataEntry.text(
-                                text=node_timing.completed_at.isoformat(timespec="seconds"),
-                                label="Execution Completed At",
-                            ),
-                            EventMetadataEntry.text(
-                                text=str(node_timing.duration), label="Execution Duration"
-                            ),
-                        ]
-                        entries.extend(execution_entries)
-                    if node_timing.name == "compile":
-                        execution_entries = [
-                            EventMetadataEntry.text(
-                                text=node_timing.started_at.isoformat(timespec="seconds"),
-                                label="Compilation Started At",
-                            ),
-                            EventMetadataEntry.text(
-                                text=node_timing.completed_at.isoformat(timespec="seconds"),
-                                label="Compilation Completed At",
-                            ),
-                            EventMetadataEntry.text(
-                                text=str(node_timing.duration), label="Compilation Duration"
-                            ),
-                        ]
-                        entries.extend(execution_entries)
+def _poll_rpc(
+    context: SolidExecutionContext, request_token: str, should_yield_materializations: bool = True
+) -> DbtRpcOutput:
+    """Polls the dbt RPC server for the status of a request until the state is ``success``."""
+    from ..utils import generate_materializations
 
-                yield AssetMaterialization(
-                    description="A materialized node within the dbt graph.",
-                    metadata_entries=entries,
-                    asset_key=node_result.node["unique_id"],
-                )
-
-
-def dbt_rpc_poll(
-    context, request_token: str, should_yield_materializations: bool = True
-) -> DbtRpcPollResult:
-
-    resp = context.resources.dbt_rpc.poll(
-        request_token=request_token, logs=context.solid_config["logs"]
-    )
-    raise_for_rpc_error(context, resp)
-
-    interval = context.solid_config["interval"]
     logs_start = 0
-    while resp.json().get("result").get("state") == "running":
-        context.log.debug(
-            f"Request {request_token} currently in state '{resp.json().get('result').get('state')}' (elapsed time {resp.json().get('result').get('elapsed', 0)} seconds). Sleeping for {interval}s.."
-        )
+    interval = context.solid_config.get("interval")
 
-        if context.solid_config["logs"]:
-            logs = resp.json().get("result").get("logs")
-            if len(logs) > 0:
-                log_rpc(context, logs)
-            logs_start += len(logs)
+    elapsed_time = -1
+    current_state = None
 
-        time.sleep(interval)
+    while True:
+        # Poll for the dbt RPC request.
+        context.log.debug(f"RequestToken: {request_token}")
         resp = context.resources.dbt_rpc.poll(
             request_token=request_token, logs=context.solid_config["logs"], logs_start=logs_start
         )
         raise_for_rpc_error(context, resp)
 
-    if resp.json().get("result").get("state") != "success":
+        resp_json = resp.json()
+        resp_result = resp_json.get("result", {})
+
+        # Pass dbt RPC logs into the Dagster/Dagit logger.
         if context.solid_config["logs"]:
-            logs = resp.json().get("result").get("logs")
+            logs = resp_result.get("logs", [])
             if len(logs) > 0:
                 log_rpc(context, logs)
+            logs_start += len(logs)
 
+        current_state = resp_result.get("state")
+        # Stop polling if request's state is no longer "running".
+        if current_state != "running":
+            break
+
+        elapsed_time = resp_result.get("elapsed", 0)
+        # Sleep for the configured time interval before polling again.
+        context.log.debug(
+            f"Request {request_token} currently in state '{current_state}' (elapsed time "
+            f"{elapsed_time} seconds). Sleeping for {interval}s..."
+        )
+        time.sleep(interval)
+
+    if current_state != "success":
         raise Failure(
-            description=f"Request {request_token} finished with state '{resp.json().get('result').get('state')}' in {resp.json().get('result').get('elapsed')} seconds",
+            description=(
+                f"Request {request_token} finished with state '{current_state}' in "
+                f"{elapsed_time} seconds"
+            ),
         )
 
-    else:
-        context.log.info(
-            f"Request {request_token} finished with state '{resp.json().get('result').get('state')}' in {resp.json().get('result').get('elapsed')} seconds"
-        )
+    context.log.info(
+        f"Request {request_token} finished with state '{current_state}' in {elapsed_time} seconds"
+    )
+    context.log.debug(json.dumps(resp_result, indent=2))
 
-    if context.solid_config["logs"]:
-        logs = resp.json().get("result").get("logs")
-        if len(logs) > 0:
-            log_rpc(context, logs)
+    polled_run_results = DbtRpcOutput.from_dict(resp_result)
 
-    context.log.debug(json.dumps(resp.json().get("result"), indent=2))
-    rpc_poll_result = DbtRpcPollResult.from_results(resp.json().get("result"))
     if should_yield_materializations:
-        for materialization in generate_materializations(rpc_poll_result=rpc_poll_result):
+        for materialization in generate_materializations(polled_run_results):
             yield materialization
-    yield Output(value=rpc_poll_result, output_name="result")
+
+    yield Output(polled_run_results)
 
 
-def unwrap_result(dbt_rpc_poll_generator) -> DbtRpcPollResult:
-    """A helper function that extracts the `DbtRpcPollResult` value from a generator.
+def unwrap_result(poll_rpc_generator) -> DbtRpcOutput:
+    """A helper function that extracts the `DbtRpcOutput` value from a generator.
 
-    The parameter `dbt_rpc_poll_generator` is expected to be an invocation of `dbt_rpc_poll`.
+    The parameter `poll_rpc_generator` is expected to be an invocation of `_poll_rpc`.
     """
     output = None
-    for x in dbt_rpc_poll_generator:
+    for x in poll_rpc_generator:
         output = x
 
     if output is None:
-        raise DagsterDbtUnexpectedRpcPollOutput(
-            description="dbt_rpc_poll yielded None as its last value. Expected value of type Output containing DbtRpcPollResult.",
+        raise DagsterDbtRpcUnexpectedPollOutputError(
+            description="poll_rpc yielded None as its last value. Expected value of type Output containing DbtRpcOutput.",
         )
 
     if not isinstance(output, Output):
-        raise DagsterDbtUnexpectedRpcPollOutput(
-            description=f"dbt_rpc_poll yielded value of type {type(output)} as its last value. Expected value of type Output containing DbtRpcPollResult.",
+        raise DagsterDbtRpcUnexpectedPollOutputError(
+            description=f"poll_rpc yielded value of type {type(output)} as its last value. Expected value of type Output containing DbtRpcOutput.",
         )
 
-    if not isinstance(output.value, DbtRpcPollResult):
-        raise DagsterDbtUnexpectedRpcPollOutput(
-            description=f"dbt_rpc_poll yielded Output containing {type(output.value)}. Expected DbtRpcPollResult.",
+    if not isinstance(output.value, DbtRpcOutput):
+        raise DagsterDbtRpcUnexpectedPollOutputError(
+            description=f"poll_rpc yielded Output containing {type(output.value)}. Expected DbtRpcOutput.",
         )
 
     return output.value
@@ -202,7 +148,12 @@ def unwrap_result(dbt_rpc_poll_generator) -> DbtRpcPollResult:
     required_resource_keys={"dbt_rpc"},
     tags={"kind": "dbt"},
 )
-def dbt_rpc_run(context) -> String:
+def dbt_rpc_run(context: SolidExecutionContext) -> String:
+    """This solid sends the ``dbt run`` command to a dbt RPC server and returns the request token.
+
+    This dbt RPC solid is asynchronous. The request token can be used in subsequent RPC requests to
+    poll the progress of the running dbt process.
+    """
     resp = context.resources.dbt_rpc.run(
         models=context.solid_config["models"], exclude=context.solid_config["exclude"]
     )
@@ -214,7 +165,7 @@ def dbt_rpc_run(context) -> String:
 @solid(
     description="A solid to invoke dbt run over RPC and poll the resulting RPC process until it's complete.",
     input_defs=[InputDefinition(name="start_after", dagster_type=Nothing)],
-    output_defs=[OutputDefinition(name="result", dagster_type=DbtRpcPollResult)],
+    output_defs=[OutputDefinition(name="result", dagster_type=DbtRpcOutput)],
     config_schema={
         "models": Field(
             config=Noneable(Array(String)),
@@ -261,12 +212,26 @@ def dbt_rpc_run(context) -> String:
         "task_tags": Permissive(),
         "max_retries": Field(config=Int, is_required=False, default_value=5),
         "retry_interval": Field(config=Int, is_required=False, default_value=120),
+        "yield_materializations": Field(
+            config=Bool,
+            is_required=False,
+            default_value=True,
+            description=(
+                "If True, materializations corresponding to the results of the dbt operation will "
+                "be yielded when the solid executes. Default: True"
+            ),
+        ),
     },
     required_resource_keys={"dbt_rpc"},
     tags={"kind": "dbt"},
 )
-def dbt_rpc_run_and_wait(context) -> DbtRpcPollResult:
+def dbt_rpc_run_and_wait(context: SolidExecutionContext) -> DbtRpcOutput:
+    """This solid sends the ``dbt run`` command to a dbt RPC server and returns the result of the
+    executed dbt process.
 
+    This dbt RPC solid is synchronous, and will periodically poll the dbt RPC server until the dbt
+    process is completed.
+    """
     if context.solid_config["task_tags"]:
         results = context.resources.dbt_rpc.ps().json()
         for task in results["result"]["rows"]:
@@ -305,7 +270,11 @@ def dbt_rpc_run_and_wait(context) -> DbtRpcPollResult:
     context.log.debug(resp.text)
     raise_for_rpc_error(context, resp)
     request_token = resp.json().get("result").get("request_token")
-    return dbt_rpc_poll(context, request_token)
+    return _poll_rpc(
+        context,
+        request_token,
+        should_yield_materializations=context.solid_config["yield_materializations"],
+    )
 
 
 @solid(
@@ -347,7 +316,12 @@ def dbt_rpc_run_and_wait(context) -> DbtRpcPollResult:
     required_resource_keys={"dbt_rpc"},
     tags={"kind": "dbt"},
 )
-def dbt_rpc_test(context) -> String:
+def dbt_rpc_test(context: SolidExecutionContext) -> String:
+    """This solid sends the ``dbt test`` command to a dbt RPC server and returns the request token.
+
+    This dbt RPC solid is asynchronous. The request token can be used in subsequent RPC requests to
+    poll the progress of the running dbt process.
+    """
     resp = context.resources.dbt_rpc.test(
         models=context.solid_config["models"],
         exclude=context.solid_config["exclude"],
@@ -360,9 +334,12 @@ def dbt_rpc_test(context) -> String:
 
 
 @solid(
-    description="A solid to invoke dbt test over RPC and poll the resulting RPC process until it's complete.",
+    description=(
+        "A solid to invoke dbt test over RPC and poll the resulting RPC process until it's "
+        "complete."
+    ),
     input_defs=[InputDefinition(name="start_after", dagster_type=Nothing)],
-    output_defs=[OutputDefinition(name="result", dagster_type=DbtRpcPollResult)],
+    output_defs=[OutputDefinition(name="result", dagster_type=DbtRpcOutput)],
     config_schema={
         "models": Field(
             config=Noneable(Array(String)),
@@ -400,11 +377,26 @@ def dbt_rpc_test(context) -> String:
             default_value=True,
             description="Whether or not to return logs from the process.",
         ),
+        "yield_materializations": Field(
+            config=Bool,
+            is_required=False,
+            default_value=True,
+            description=(
+                "If True, materializations corresponding to the results of the dbt operation will "
+                "be yielded when the solid executes. Default: True"
+            ),
+        ),
     },
     required_resource_keys={"dbt_rpc"},
     tags={"kind": "dbt"},
 )
-def dbt_rpc_test_and_wait(context) -> DbtRpcPollResult:
+def dbt_rpc_test_and_wait(context: SolidExecutionContext) -> DbtRpcOutput:
+    """This solid sends the ``dbt test`` command to a dbt RPC server and returns the result of the
+    executed dbt process.
+
+    This dbt RPC solid is synchronous, and will periodically poll the dbt RPC server until the dbt
+    process is completed.
+    """
     resp = context.resources.dbt_rpc.test(
         models=context.solid_config["models"],
         exclude=context.solid_config["exclude"],
@@ -414,7 +406,11 @@ def dbt_rpc_test_and_wait(context) -> DbtRpcPollResult:
     context.log.debug(resp.text)
     raise_for_rpc_error(context, resp)
     request_token = resp.json().get("result").get("request_token")
-    return dbt_rpc_poll(context, request_token)
+    return _poll_rpc(
+        context,
+        request_token,
+        should_yield_materializations=context.solid_config["yield_materializations"],
+    )
 
 
 @solid(
@@ -443,7 +439,13 @@ def dbt_rpc_test_and_wait(context) -> DbtRpcPollResult:
     required_resource_keys={"dbt_rpc"},
     tags={"kind": "dbt"},
 )
-def dbt_rpc_run_operation(context) -> String:
+def dbt_rpc_run_operation(context: SolidExecutionContext) -> String:
+    """This solid sends the ``dbt run-operation`` command to a dbt RPC server and returns the
+    request token.
+
+    This dbt RPC solid is asynchronous. The request token can be used in subsequent RPC requests to
+    poll the progress of the running dbt process.
+    """
     resp = context.resources.dbt_rpc.run_operation(
         macro=context.solid_config["macro"], args=context.solid_config["args"]
     )
@@ -453,9 +455,12 @@ def dbt_rpc_run_operation(context) -> String:
 
 
 @solid(
-    description="A solid to invoke a dbt run operation over RPC and poll the resulting RPC process until it's complete.",
+    description=(
+        "A solid to invoke a dbt run operation over RPC and poll the resulting RPC process until "
+        "it's complete."
+    ),
     input_defs=[InputDefinition(name="start_after", dagster_type=Nothing)],
-    output_defs=[OutputDefinition(name="result", dagster_type=DbtRpcPollResult)],
+    output_defs=[OutputDefinition(name="result", dagster_type=DbtRpcOutput)],
     config_schema={
         "macro": Field(
             config=String,
@@ -480,18 +485,37 @@ def dbt_rpc_run_operation(context) -> String:
             default_value=True,
             description="Whether or not to return logs from the process.",
         ),
+        "yield_materializations": Field(
+            config=Bool,
+            is_required=False,
+            default_value=True,
+            description=(
+                "If True, materializations corresponding to the results of the dbt operation will "
+                "be yielded when the solid executes. Default: True"
+            ),
+        ),
     },
     required_resource_keys={"dbt_rpc"},
     tags={"kind": "dbt"},
 )
-def dbt_rpc_run_operation_and_wait(context) -> DbtRpcPollResult:
+def dbt_rpc_run_operation_and_wait(context: SolidExecutionContext) -> DbtRpcOutput:
+    """This solid sends the ``dbt run-operation`` command to a dbt RPC server and returns the
+    result of the executed dbt process.
+
+    This dbt RPC solid is synchronous, and will periodically poll the dbt RPC server until the dbt
+    process is completed.
+    """
     resp = context.resources.dbt_rpc.run_operation(
         macro=context.solid_config["macro"], args=context.solid_config["args"]
     )
     context.log.debug(resp.text)
     raise_for_rpc_error(context, resp)
     request_token = resp.json().get("result").get("request_token")
-    return dbt_rpc_poll(context, request_token)
+    return _poll_rpc(
+        context,
+        request_token,
+        should_yield_materializations=context.solid_config["yield_materializations"],
+    )
 
 
 @solid(
@@ -521,7 +545,13 @@ def dbt_rpc_run_operation_and_wait(context) -> DbtRpcPollResult:
     required_resource_keys={"dbt_rpc"},
     tags={"kind": "dbt"},
 )
-def dbt_rpc_snapshot(context) -> String:
+def dbt_rpc_snapshot(context: SolidExecutionContext) -> String:
+    """This solid sends the ``dbt snapshot`` command to a dbt RPC server and returns the
+    request token.
+
+    This dbt RPC solid is asynchronous. The request token can be used in subsequent RPC requests to
+    poll the progress of the running dbt process.
+    """
     resp = context.resources.dbt_rpc.snapshot(
         select=context.solid_config["select"], exclude=context.solid_config["exclude"]
     )
@@ -531,9 +561,12 @@ def dbt_rpc_snapshot(context) -> String:
 
 
 @solid(
-    description="A solid to invoke a dbt snapshot over RPC and poll the resulting RPC process until it's complete.",
+    description=(
+        "A solid to invoke a dbt snapshot over RPC and poll the resulting RPC process until "
+        "it's complete."
+    ),
     input_defs=[InputDefinition(name="start_after", dagster_type=Nothing)],
-    output_defs=[OutputDefinition(name="result", dagster_type=DbtRpcPollResult)],
+    output_defs=[OutputDefinition(name="result", dagster_type=DbtRpcOutput)],
     config_schema={
         "select": Field(
             config=Noneable(Array(String)),
@@ -562,12 +595,26 @@ def dbt_rpc_snapshot(context) -> String:
         "task_tags": Permissive(),
         "max_retries": Field(config=Int, is_required=False, default_value=5),
         "retry_interval": Field(config=Int, is_required=False, default_value=120),
+        "yield_materializations": Field(
+            config=Bool,
+            is_required=False,
+            default_value=True,
+            description=(
+                "If True, materializations corresponding to the results of the dbt operation will "
+                "be yielded when the solid executes. Default: True"
+            ),
+        ),
     },
     required_resource_keys={"dbt_rpc"},
     tags={"kind": "dbt"},
 )
-def dbt_rpc_snapshot_and_wait(context) -> DbtRpcPollResult:
+def dbt_rpc_snapshot_and_wait(context: SolidExecutionContext) -> DbtRpcOutput:
+    """This solid sends the ``dbt snapshot`` command to a dbt RPC server and returns the result of
+    the executed dbt process.
 
+    This dbt RPC solid is synchronous, and will periodically poll the dbt RPC server until the dbt
+    process is completed.
+    """
     if context.solid_config["task_tags"]:
         results = context.resources.dbt_rpc.ps().json()
         for task in results["result"]["rows"]:
@@ -586,7 +633,11 @@ def dbt_rpc_snapshot_and_wait(context) -> DbtRpcPollResult:
     context.log.debug(resp.text)
     raise_for_rpc_error(context, resp)
     request_token = resp.json().get("result").get("request_token")
-    return dbt_rpc_poll(context, request_token)
+    return _poll_rpc(
+        context,
+        request_token,
+        should_yield_materializations=context.solid_config["yield_materializations"],
+    )
 
 
 @solid(
@@ -616,7 +667,13 @@ def dbt_rpc_snapshot_and_wait(context) -> DbtRpcPollResult:
     required_resource_keys={"dbt_rpc"},
     tags={"kind": "dbt"},
 )
-def dbt_rpc_snapshot_freshness(context) -> String:
+def dbt_rpc_snapshot_freshness(context: SolidExecutionContext) -> String:
+    """This solid sends the ``dbt source snapshot-freshness`` command to a dbt RPC server and
+    returns the request token.
+
+    This dbt RPC solid is asynchronous. The request token can be used in subsequent RPC requests to
+    poll the progress of the running dbt process.
+    """
     command = ""
 
     if context.solid_config["warn_error"]:
@@ -636,9 +693,12 @@ def dbt_rpc_snapshot_freshness(context) -> String:
 
 
 @solid(
-    description="A solid to invoke dbt source snapshot-freshness over RPC and poll the resulting RPC process until it's complete.",
+    description=(
+        "A solid to invoke dbt source snapshot-freshness over RPC and poll the resulting "
+        "RPC process until it's complete."
+    ),
     input_defs=[InputDefinition(name="start_after", dagster_type=Nothing)],
-    output_defs=[OutputDefinition(name="result", dagster_type=DbtRpcPollResult)],
+    output_defs=[OutputDefinition(name="result", dagster_type=DbtRpcOutput)],
     config_schema={
         "select": Field(
             config=Noneable(Array(String)),
@@ -664,11 +724,26 @@ def dbt_rpc_snapshot_freshness(context) -> String:
             default_value=True,
             description="Whether or not to return logs from the process.",
         ),
+        "yield_materializations": Field(
+            config=Bool,
+            is_required=False,
+            default_value=True,
+            description=(
+                "If True, materializations corresponding to the results of the dbt operation will "
+                "be yielded when the solid executes. Default: True"
+            ),
+        ),
     },
     required_resource_keys={"dbt_rpc"},
     tags={"kind": "dbt"},
 )
-def dbt_rpc_snapshot_freshness_and_wait(context) -> DbtRpcPollResult:
+def dbt_rpc_snapshot_freshness_and_wait(context: SolidExecutionContext) -> DbtRpcOutput:
+    """This solid sends the ``dbt source snapshot`` command to a dbt RPC server and returns the
+    result of the executed dbt process.
+
+    This dbt RPC solid is synchronous, and will periodically poll the dbt RPC server until the dbt
+    process is completed.
+    """
     command = ""
 
     if context.solid_config["warn_error"]:
@@ -685,7 +760,11 @@ def dbt_rpc_snapshot_freshness_and_wait(context) -> DbtRpcPollResult:
     context.log.debug(resp.text)
     raise_for_rpc_error(context, resp)
     request_token = resp.json().get("result").get("request_token")
-    return dbt_rpc_poll(context, request_token)
+    return _poll_rpc(
+        context,
+        request_token,
+        should_yield_materializations=context.solid_config["yield_materializations"],
+    )
 
 
 @solid(
@@ -713,33 +792,61 @@ def dbt_rpc_snapshot_freshness_and_wait(context) -> DbtRpcPollResult:
             default_value=True,
             description="Whether or not to return logs from the process.",
         ),
+        "yield_materializations": Field(
+            config=Bool,
+            is_required=False,
+            default_value=True,
+            description=(
+                "If True, materializations corresponding to the results of the dbt operation will "
+                "be yielded when the solid executes. Default: True"
+            ),
+        ),
     },
     required_resource_keys={"dbt_rpc"},
     tags={"kind": "dbt"},
 )
-def dbt_rpc_compile_sql(context, sql: String) -> String:
+def dbt_rpc_compile_sql(context: SolidExecutionContext, sql: String) -> String:
+    """This solid sends the ``dbt compile`` command to a dbt RPC server and returns the request
+    token.
+
+    This dbt RPC solid is asynchronous. The request token can be used in subsequent RPC requests to
+    poll the progress of the running dbt process.
+    """
     resp = context.resources.dbt_rpc.compile_sql(sql=sql, name=context.solid_config["name"])
     context.log.debug(resp.text)
     raise_for_rpc_error(context, resp)
     request_token = resp.json().get("result").get("request_token")
-    result = unwrap_result(dbt_rpc_poll(context, request_token))
+    result = unwrap_result(
+        _poll_rpc(
+            context,
+            request_token,
+            should_yield_materializations=context.solid_config["yield_materializations"],
+        )
+    )
     return result.results[0].node["compiled_sql"]
 
 
 def create_dbt_rpc_run_sql_solid(
     name: str, output_def: Optional[OutputDefinition] = None, **kwargs
 ) -> Callable:
-    """This function is a factory which constructs a solid that will copy the results of a SQL query run within the context of a dbt project to a DataFrame.
+    """This function is a factory which constructs a solid that will copy the results of a SQL query
+    run within the context of a dbt project to a pandas ``DataFrame``.
 
-    Any kwargs passed to this function will be passed along to the underlying @solid decorator.
-    However, note that overriding config, input_defs, and required_resource_keys is not supported. You might consider using
-    @composite_solid to wrap this solid in the cases where you'd like to configure the solid
-    with different config fields.
+    Any kwargs passed to this function will be passed along to the underlying :func:`@solid
+    <dagster.solid>` decorator. However, note that overriding ``config_schema``, ``input_defs``, and
+    ``required_resource_keys`` is not allowed and will throw a :class:`DagsterInvalidDefinitionError
+    <dagster.DagsterInvalidDefinitionError>`.
+
+    If you would like to configure this solid with different config fields, you could consider using
+    :func:`@composite_solid <dagster.composite_solid>` to wrap this solid.
 
     Args:
         name (str): The name of this solid.
-        output_def (OutputDefinition, optional): The OutputDefinition for the solid. This value should always be a representation
-            of a pandas DataFrame. If not specificed, the solid will default to an OutputDefinition named "df" with a DataFrame dagster type.
+        output_def (OutputDefinition, optional): The :class:`OutputDefinition
+            <dagster.OutputDefinition>` for the solid. This value should always be a representation
+            of a pandas ``DataFrame``. If not specified, the solid will default to an
+            :class:`OutputDefinition <dagster.OutputDefinition>` named "df" with a ``DataFrame``
+            dagster type.
 
     Returns:
         SolidDefinition: Returns the constructed solid definition.
@@ -760,7 +867,8 @@ def create_dbt_rpc_run_sql_solid(
         name=name,
         description=kwargs.pop(
             "description",
-            "A solid to run a SQL query in context of a dbt project over RPC and return the results in a pandas DataFrame.",
+            "A solid to run a SQL query in context of a dbt project over RPC and return the "
+            "results in a pandas DataFrame.",
         ),
         input_defs=[
             InputDefinition(name="start_after", dagster_type=Nothing),
@@ -788,17 +896,32 @@ def create_dbt_rpc_run_sql_solid(
                 default_value=True,
                 description="Whether or not to return logs from the process.",
             ),
+            "yield_materializations": Field(
+                config=Bool,
+                is_required=False,
+                default_value=True,
+                description=(
+                    "If True, materializations corresponding to the results of the dbt operation "
+                    "will be yielded when the solid executes. Default: True"
+                ),
+            ),
         },
         required_resource_keys={"dbt_rpc"},
         tags={"kind": "dbt"},
         **kwargs,
     )
-    def _dbt_rpc_run_sql(context, sql: String) -> DataFrame:
+    def _dbt_rpc_run_sql(context: SolidExecutionContext, sql: String) -> DataFrame:
         resp = context.resources.dbt_rpc.run_sql(sql=sql, name=context.solid_config["name"])
         context.log.debug(resp.text)
         raise_for_rpc_error(context, resp)
         request_token = resp.json().get("result").get("request_token")
-        result = unwrap_result(dbt_rpc_poll(context, request_token))
+        result = unwrap_result(
+            _poll_rpc(
+                context,
+                request_token,
+                should_yield_materializations=context.solid_config["yield_materializations"],
+            )
+        )
         table = result.results[0].table
         return pd.DataFrame.from_records(data=table["rows"], columns=table["column_names"])
 
